import { Hex, SignTypedDataParameters, PublicClient } from 'viem';
import { EventEmitter } from 'events';
import { Chain as Chain$1 } from 'viem/chains';
import { Connector } from '@wagmi/connectors';
import { WalletClient } from 'wagmi';

type WalletEventName = "connect" | "disconnect" | "action";
type WalletMessage = {
    appId: string;
    target: "CyberWallet" | "CyberApp";
    event: WalletEvent;
    messageId: string;
    appInfo?: AppInfo;
};
declare enum EventType {
    Request = "request",
    Response = "response"
}
type EventData = {
    method: string;
    type?: EventType;
    data?: any;
    error?: any;
};
type WalletEvent = {
    name: WalletEventName;
    data?: EventData;
};
type Address = `0x${string}`;
type Token = {
    contract: Address;
    decimals: number;
};
type WalletTransaction = {
    from?: Address;
    to: Address;
    value: string;
    data: Hex;
    ctx: {
        chainId: number;
        owner: Address;
        appId: string;
    };
};
type SendTransaction = (transaction: WalletTransaction, option?: {
    description?: string;
}) => Promise<any>;
type AppInfo = {
    appId: string;
    name: string;
    icon: string;
};
declare enum ErrorType {
    SendTransactionError = "SendTransactionError"
}
interface EIP1193Provider extends EventEmitter {
    request(args: {
        method: string;
        params?: any[] | unknown;
    }): Promise<unknown>;
}
interface EIP1193ProviderError extends Error {
    code: number;
    data?: unknown;
}

declare class CyberAccount {
    address: Address;
    ownerAddress: Address;
    constructor({ address, ownerAddress, }: {
        address: Address;
        ownerAddress: Address;
    });
}

type MessengerParams = {
    walletWindow: Window;
    appInfo?: AppInfo;
};
declare class Messenger {
    walletWindow: Window;
    appWindow: Window;
    appInfo?: AppInfo;
    constructor({ walletWindow, appInfo }: MessengerParams);
    sendMessage(event: WalletEvent): `${string}-${string}-${string}-${string}-${string}`;
    onMessage(callback: (message: WalletMessage) => void): void;
    private postMessage;
}

declare class Chain {
    id: number;
    sendTransactionBase: SendTransaction;
    cyberAccount: CyberAccount | null;
    appId: string;
    constructor({ id, sendTransaction, appId, }: {
        id: number;
        sendTransaction: SendTransaction;
        appId: string;
    });
    setCyberAccount(cyberAccount: CyberAccount | null): void;
    sendTransaction(transaction: Omit<WalletTransaction, "ctx">, option?: {
        description?: string;
    }): Promise<any>;
}

type CyberWalletParams = {
    contextWindow: Window;
    appInfo: AppInfo;
};
declare class CyberWallet {
    contextWindow: Window;
    connected: boolean;
    messenger: Messenger;
    cyberAccount: CyberAccount | null;
    optimism: Chain;
    optimismGoerli: Chain;
    polygon: Chain;
    polygonMumbai: Chain;
    arbitrum: Chain;
    arbitrumGoerli: Chain;
    linea: Chain;
    lineaTestnet: Chain;
    base: Chain;
    baseGoerli: Chain;
    opBnb: Chain;
    scrollSepolia: Chain;
    scroll: Chain;
    mainnet: Chain;
    bsc: Chain;
    bscTestnet: Chain;
    constructor({ contextWindow, appInfo }: CyberWalletParams);
    private bindedSendTransaction;
    setConnection(connected: boolean): void;
    setCyberAccount(cyberAccount: CyberAccount | null): void;
    private resolveActionResponse;
    sendTransaction(transaction: WalletTransaction): Promise<unknown>;
    signMessage(message: string): Promise<unknown>;
    signTypedData(typedData: SignTypedDataParameters): Promise<unknown>;
    getAuthToken(): Promise<string>;
}

declare class CyberApp {
    appId: string;
    name: string;
    cyberWallet: CyberWallet;
    messenger: Messenger;
    icon: string;
    constructor({ appId, name, icon }: AppInfo);
    start(): Promise<CyberAccount>;
    connect(): Promise<CyberAccount>;
}

declare class EventError extends Error {
    details: string;
    shortMessage: string;
    constructor({ name, details, shortMessage, }: {
        name?: string;
        details: string;
        shortMessage?: string;
    });
}

declare class CyberProvider extends EventEmitter implements EIP1193Provider {
    name: "CyberWallet";
    cyberApp: CyberApp;
    chainId: number;
    readonly isMetaMask: any;
    _events: {
        connect?: () => void;
    };
    chain?: Chain;
    publicClient?: PublicClient;
    constructor({ app, chainId }: {
        app: CyberApp;
        chainId: number;
    });
    setPublicClient(chainId: number): PublicClient | undefined;
    connect(): Promise<void>;
    disconnect(): Promise<void>;
    private getChainKeyByChainId;
    private getChainByChainId;
    request(request: {
        method: string;
        params?: any;
    }): Promise<any>;
}

type CyberWalletConnectorOptions = {
    name?: string;
    icon?: string;
    appId?: string;
    shimDisconnect?: boolean;
};
declare class CyberWalletConnector extends Connector<CyberProvider, CyberWalletConnectorOptions> {
    provider?: CyberProvider;
    readonly id = "cyberwallet";
    readonly name = "CyberWallet";
    ready: boolean;
    protected shimDisconnectKey: string;
    constructor({ chains, options: options_, }: {
        chains?: Chain$1[];
        options?: CyberWalletConnectorOptions;
    });
    connect({ chainId }?: {
        chainId?: number;
    }): Promise<{
        account: `0x${string}`;
        chain: {
            id: number;
            unsupported: boolean;
        };
    }>;
    disconnect(): Promise<void>;
    getAccount(): Promise<`0x${string}`>;
    getChainId(): Promise<number>;
    getProvider(): Promise<CyberProvider>;
    getWalletClient({ chainId, }?: {
        chainId?: number;
    }): Promise<WalletClient>;
    isAuthorized(): Promise<boolean>;
    switchChain(chainId: number): Promise<Chain$1>;
    protected onAccountsChanged: (accounts: string[]) => void;
    isChainUnsupported(chainId: number): boolean;
    protected onChainChanged: (chainId: number | string) => void;
    protected onDisconnect(): void;
    protected isUserRejectedRequestError(error: unknown): boolean;
}

declare const isCyberWallet: () => boolean;
declare function isChainUnsupported(chainId: number): boolean;
declare function getSignerAddress(): any;

export { type Address, type AppInfo, CyberApp, CyberProvider, CyberWallet, CyberWalletConnector, type EIP1193Provider, type EIP1193ProviderError, ErrorType, type EventData, EventError, EventType, type SendTransaction, type Token, type WalletEvent, type WalletEventName, type WalletMessage, type WalletTransaction, getSignerAddress, isChainUnsupported, isCyberWallet };
